var logger = org.slf4j.LoggerFactory.getLogger("useruser.js");
var lk = org.grouplens.lenskit;
var dataset = properties.get("dataset");

recipe.addEval("Coverage");
recipe.addEval("NDCG");
recipe.addEval("HLUtility");
recipe.addEval("MAE");
recipe.addEval("RMSE");

function extend(parent, child) {
  for (k in parent) {
    child[k] = parent[k];
  }
  return child;
}

importPackage(org.grouplens.lenskit);
importPackage(org.grouplens.lenskit.params);
importPackage(org.grouplens.lenskit.norm);
importPackage(org.grouplens.lenskit.baseline);
importPackage(org.grouplens.lenskit.knn);
importPackage(org.grouplens.lenskit.knn.params);
importPackage(org.grouplens.lenskit.knn.item);
importPackage(org.grouplens.lenskit.knn.user);
	
logger.info("Configuring user-user run for {}", dataset);


var nss = [10, 20, 30, 50, 75, 100];
function NbrSize(n) {
  this.attrs = {nnbrs: n}
  this.nnbrs = n
}
NbrSize.prototype.apply = function(rec) {
  rec.factory.set(NeighborhoodSize, this.nnbrs);
};


var sims = ["Cosine", "CosineNorm", "Pearson", "Spearman"];
function Sim(sim, adj) {
  this.attrs = {sim: sim};
  this.norm = IdentityVectorNormalizer;
  
  var baseSim;
  if (/^Cosine/.test(sim)) {
	  baseSim = lk.knn.CosineSimilarity;
	  if (sim == "CosineNorm") {
		  this.norm = BaselineSubtractingNormalizer;
		  this.baseline = UserMeanPredictor;
	  }
  } else if (sim == "Pearson") {
	  baseSim = PearsonCorrelation;
  } else if (sim == "Spearman") {
	  baseSim = SpearmanRankCorrelation;
  } else {
	  logger.error("Invalid similarity function {}", sim);
	  throw {error: "invalid similarity", arg: sim}
  }
  
  if (adj == "shrink") {
    this.attrs.Adj = "Shrink 100";
    this.simDamping = 100;
    this.sim = baseSim;
  } else if (adj == "weight") {
    this.attrs.Adj = "Weight 50";
    this.simDamping = 0;
    this.weight = 50;
    this.sim = SignificanceWeight;
    this.weightedSim = baseSim;
  } else if (!adj) {
    this.attrs.Adj = "None";
    this.simDamping = 0;
    this.sim = baseSim;
  } else {
    logger.error("Invalid adjustment {}", adj);
    throw {message: "Invalid adjustment", reason: adj};
  }
}

Sim.prototype.apply = function(rec) {
  rec.factory.setComponent(UserSimilarity, Similarity, this.sim);
  rec.factory.set(SimilarityDamping, this.simDamping);
  if (this.weightedSim) {
    rec.factory.set(WeightThreshold, this.weight);
    rec.factory.setComponent(WeightedSimilarity, Similarity, this.weightedSim);
  }
  if (this.norm)
    rec.factory.setComponent(VectorNormalizer, this.norm);
  if (this.baseline)
    rec.factory.setComponent(NormalizerBaseline, BaselinePredictor, this.baseline);
}


function findBaseline(bl) {
  if (bl == "None")
    return null;
  else
    return lk.baseline[bl + "Predictor"];
}


var norms = ["None", "GlobalMean", "UserMean",
             "Variance", "ItemMean", "ItemUserMean"];
function Norm(norm) {
  this.attrs = {Norm: norm}
  if (norm == "None")
    this.norm = IdentityVectorNormalizer;
  else if (norm == "Variance")
    this.norm = MeanVarianceNormalizer;
  else
    this.baseline = findBaseline(norm);
}
Norm.prototype = {
  apply: function(rec) {
    if (this.norm) {
      rec.factory.setComponent(PredictNormalizer, VectorNormalizer, this.norm);
    } else {
      // FIXME: I think this overlaps with the CosineNorm, we need a 
      // way to bind two different normalizers and baselines
      rec.factory.setComponent(PredictNormalizer, Normalizer, BaselineSubtractingNormalizer);
      rec.factory.setComponent(NormalizerBaseline, BaselinePredictor, this.baseline);
    }
  }
};

function Baseline(bl) {
  this.attrs = { Baseline: bl }
  this.baseline = findBaseline(bl);
}
Baseline.prototype = {
  apply: function (rec) {
	  rec.factory.setComponent(BaselinePredictor, this.baseline);
  }
};

function UserUser() {}
UserUser.prototype = {
  apply: function(rec) {
    rec.factory.set(MeanSmoothing, 0);

    rec.factory.setComponent(RatingPredictor, UserUserRatingPredictor);
    // rec.factory.setComponent(ItemRecommender, UserUserRatingRecommender);
  }
}

function makeTrial(trial) {
	rec = recipe.addAlgorithm();
	rec.name = "UserUser";
	rec.preload = true;
	
	for (var f in trial) {
		var t = trial[f];
		t.apply(rec);
		for (k in t.attrs) {
			rec.attributes.put(k, t.attrs[k]);
		}
	}
}

// Compute the neighborhood size trials
if (dataset == "ml-10m") {
  nss = [30];
  sims = ["CosineNorm", "Pearson"];
}

nss.forEach(function(n) {
  var trial = {
    nnbrs: new NbrSize(n),
    norm: new Norm("Variance"),
    baseline: new Baseline("ItemUserMean"),
    algo: new UserUser()
  };
  sims.forEach(function(s) {
    if (dataset != "ml-10m") {
    makeTrial(extend(trial, {
      similarity: new Sim(s)
    }));
    }
    if (dataset != "ml-10m" || s == "Pearson") {
    makeTrial(extend(trial, {
      similarity: new Sim(s, "weight")
    }));
    }
    if (dataset != "ml-10m" || s == "CosineNorm") {
    makeTrial(extend(trial, {
      similarity: new Sim(s, "shrink")
    }));
    }
  });
});

